/*
 * This software is governed by the CeCILL-B license under French law and
 * abiding by the rules of distribution of free software. You can use, modify
 * and/ or redistribute the software under the terms of the CeCILL-B license as
 * circulated by CEA, CNRS and INRIA at the following URL "http://www.cecill.info".
 * 
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated with
 * loading, using, modifying and/or developing or reproducing the software by
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate, and that also therefore means that it is
 * reserved for developers and experienced professionals having in-depth
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling
 * the security of their systems and/or data to be ensured and, more generally,
 * to use and operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.amap.lidar.amapvox.visualization.views;

import fr.amap.commons.util.ColorGradient;
import fr.amap.lidar.amapvox.visualization.controllers.voxels.Filter;
import fr.amap.lidar.amapvox.visualization.controllers.voxels.Skyboxes;
import fr.amap.lidar.amapvox.visualization.views.jvoxelfilter.JVoxelFilter;
import fr.amap.lidar.amapvox.visualization.views.jvoxelfilter.JVoxelFilterAdapter;
import fr.amap.lidar.amapvox.visualization.views.jvoxelfilter.JVoxelFilterEvent;
import java.awt.Color;
import java.awt.Cursor;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Stack;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JLabel;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;

/**
 *
 * @author Tristan Muller (tristan.muller@cirad.fr)
 */
public class PropertiesInternalFrame extends javax.swing.JInternalFrame {

    private String[] propertyNames;
    private float minValue;
    private float maxValue;

    private final List<String> colorbarNameList;
    private final List<Color[]> colorbarList;
    private final Stack<JVoxelFilter> filters;
    private int minAndMaxGap;
    private boolean lockMinAndMax;

    /** Creates new form PropertiesInternalFrame
     * @param visualizationFrame */
    public PropertiesInternalFrame(final VisualizationFrame visualizationFrame) {
        this.visualizationFrame = visualizationFrame;
        colorbarNameList = new ArrayList();
        colorbarList = new ArrayList();
        filters = new Stack();
        minAndMaxGap = 100;
        lockMinAndMax = false;
        
        initComponents();
    }

    /** This method is called from within the constructor to initialize the
     * form. WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jTabbedPane = new javax.swing.JTabbedPane();
        jPanelPropertyTab = new javax.swing.JPanel();
        jPanelRendererProperty = new javax.swing.JPanel();
        jLabel3 = new javax.swing.JLabel();
        jComboBoxRenderedProperty = new javax.swing.JComboBox<>();
        jPanelSky = new javax.swing.JPanel();
        jLabelSky = new javax.swing.JLabel();
        jComboBoxSky = new javax.swing.JComboBox<>();
        jPanelColorbar = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        jComboBoxColorBar = new javax.swing.JComboBox<>();
        jPanelDrawingMode = new javax.swing.JPanel();
        jLabel7 = new javax.swing.JLabel();
        jSliderDrawingMode = new javax.swing.JSlider();
        jPanelColorMin = new javax.swing.JPanel();
        jLabelColorMin = new javax.swing.JLabel();
        jSliderColorMin = new javax.swing.JSlider();
        jTextFieldColorMin = new javax.swing.JTextField();
        jPanelColorMax = new javax.swing.JPanel();
        jLabelColorMax = new javax.swing.JLabel();
        jSliderColorMax = new javax.swing.JSlider();
        jTextFieldColorMax = new javax.swing.JTextField();
        jPanelLock = new javax.swing.JPanel();
        jLabelLock = new javax.swing.JLabel();
        jCheckBoxLock = new javax.swing.JCheckBox();
        jPanelFiltersTab = new javax.swing.JPanel();
        jPanel1 = new javax.swing.JPanel();
        jLabel2 = new javax.swing.JLabel();
        jScrollPaneFilters = new javax.swing.JScrollPane();
        jPanelFilters = new javax.swing.JPanel();
        jPanelResetAll = new javax.swing.JPanel();
        jButtonResetAll = new javax.swing.JButton();

        setTitle("Options");
        setToolTipText("User options panel");
        setFocusCycleRoot(false);
        setFocusable(false);
        setFrameIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/preferences-desktop.png"))); // NOI18N
        setPreferredSize(new java.awt.Dimension(450, 550));
        setRequestFocusEnabled(false);
        setVerifyInputWhenFocusTarget(false);
        setVisible(true);
        getContentPane().setLayout(new javax.swing.BoxLayout(getContentPane(), javax.swing.BoxLayout.PAGE_AXIS));

        jPanelPropertyTab.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));

        jLabel3.setFont(new java.awt.Font("DejaVu Sans", 1, 12)); // NOI18N
        jLabel3.setText("Rendered property :");
        jPanelRendererProperty.add(jLabel3);

        jComboBoxRenderedProperty.setToolTipText("Select which property of the voxel space you want to display by a gradient with the two specified colors below.");
        jComboBoxRenderedProperty.setCursor(new java.awt.Cursor(java.awt.Cursor.HAND_CURSOR));
        jComboBoxRenderedProperty.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jComboBoxRenderedPropertyActionPerformed(evt);
            }
        });
        jPanelRendererProperty.add(jComboBoxRenderedProperty);

        jPanelPropertyTab.add(jPanelRendererProperty);

        jLabelSky.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/mail-reply-sender.png"))); // NOI18N
        jLabelSky.setText("Sky (background) :");
        jPanelSky.add(jLabelSky);

        jComboBoxSky.setToolTipText("Select the sky rendered at the background");
        jComboBoxSky.setCursor(new java.awt.Cursor(java.awt.Cursor.HAND_CURSOR));
        jComboBoxSky.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jComboBoxSkyActionPerformed(evt);
            }
        });
        jPanelSky.add(jComboBoxSky);

        jPanelPropertyTab.add(jPanelSky);

        jLabel1.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/mail-reply-sender.png"))); // NOI18N
        jLabel1.setText("Color-bar :");
        jPanelColorbar.add(jLabel1);

        jComboBoxColorBar.setToolTipText("Color-bar used to display the property's values.");
        jComboBoxColorBar.setCursor(new java.awt.Cursor(java.awt.Cursor.HAND_CURSOR));
        jComboBoxColorBar.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jComboBoxColorBarActionPerformed(evt);
            }
        });
        jPanelColorbar.add(jComboBoxColorBar);

        jPanelPropertyTab.add(jPanelColorbar);

        jLabel7.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/mail-reply-sender.png"))); // NOI18N
        jLabel7.setText("Drawing mode :");
        jPanelDrawingMode.add(jLabel7);

        jSliderDrawingMode.setMajorTickSpacing(5);
        jSliderDrawingMode.setMaximum(20);
        jSliderDrawingMode.setMinorTickSpacing(1);
        jSliderDrawingMode.setPaintLabels(true);
        jSliderDrawingMode.setPaintTicks(true);
        jSliderDrawingMode.setToolTipText("Select the shape to draw the voxels (info: the POINTS mode is the faster)");
        jSliderDrawingMode.setValue(10);
        jSliderDrawingMode.setCursor(new java.awt.Cursor(java.awt.Cursor.HAND_CURSOR));
        jSliderDrawingMode.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                jSliderDrawingModeStateChanged(evt);
            }
        });
        jPanelDrawingMode.add(jSliderDrawingMode);

        jPanelPropertyTab.add(jPanelDrawingMode);

        jLabelColorMin.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/mail-reply-sender.png"))); // NOI18N
        jLabelColorMin.setText("Min value :");
        jPanelColorMin.add(jLabelColorMin);

        jSliderColorMin.setMajorTickSpacing(5);
        jSliderColorMin.setMinorTickSpacing(1);
        jSliderColorMin.setToolTipText("Select the minimum value of the current rendered property");
        jSliderColorMin.setValue(0);
        jSliderColorMin.setCursor(new java.awt.Cursor(java.awt.Cursor.HAND_CURSOR));
        jSliderColorMin.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                jSliderColorMinStateChanged(evt);
            }
        });
        jSliderColorMin.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                jSliderColorMinMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                jSliderColorMinMouseReleased(evt);
            }
        });
        jPanelColorMin.add(jSliderColorMin);

        jTextFieldColorMin.setEditable(false);
        jTextFieldColorMin.setText("0.0");
        jTextFieldColorMin.setPreferredSize(new java.awt.Dimension(100, 27));
        jPanelColorMin.add(jTextFieldColorMin);

        jPanelPropertyTab.add(jPanelColorMin);

        jLabelColorMax.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/mail-reply-sender.png"))); // NOI18N
        jLabelColorMax.setText("Max value :");
        jPanelColorMax.add(jLabelColorMax);

        jSliderColorMax.setMajorTickSpacing(5);
        jSliderColorMax.setMinorTickSpacing(1);
        jSliderColorMax.setToolTipText("Select the minimum value of the current rendered property");
        jSliderColorMax.setValue(100);
        jSliderColorMax.setCursor(new java.awt.Cursor(java.awt.Cursor.HAND_CURSOR));
        jSliderColorMax.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                jSliderColorMaxStateChanged(evt);
            }
        });
        jSliderColorMax.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                jSliderColorMaxMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                jSliderColorMaxMouseReleased(evt);
            }
        });
        jPanelColorMax.add(jSliderColorMax);

        jTextFieldColorMax.setEditable(false);
        jTextFieldColorMax.setText("1.0");
        jTextFieldColorMax.setPreferredSize(new java.awt.Dimension(100, 27));
        jPanelColorMax.add(jTextFieldColorMax);

        jPanelPropertyTab.add(jPanelColorMax);

        jLabelLock.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/mail-reply-sender.png"))); // NOI18N
        jPanelLock.add(jLabelLock);

        jCheckBoxLock.setText("Lock min and max sliders");
        jCheckBoxLock.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCheckBoxLockActionPerformed(evt);
            }
        });
        jPanelLock.add(jCheckBoxLock);

        jPanelPropertyTab.add(jPanelLock);

        jTabbedPane.addTab("Property", new javax.swing.ImageIcon(getClass().getResource("/icons/dialog-information.png")), jPanelPropertyTab, ""); // NOI18N

        jPanelFiltersTab.setLayout(new java.awt.BorderLayout());

        jLabel2.setFont(new java.awt.Font("DejaVu Sans", 1, 12)); // NOI18N
        jLabel2.setText("Filtered properties");
        jPanel1.add(jLabel2);

        jPanelFiltersTab.add(jPanel1, java.awt.BorderLayout.PAGE_START);

        jScrollPaneFilters.setHorizontalScrollBarPolicy(javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        jScrollPaneFilters.setMaximumSize(new java.awt.Dimension(32767, 50));

        jPanelFilters.setBorder(null);
        jPanelFilters.setMaximumSize(new java.awt.Dimension(0, 50));
        jPanelFilters.setLayout(new javax.swing.BoxLayout(jPanelFilters, javax.swing.BoxLayout.Y_AXIS));
        jScrollPaneFilters.setViewportView(jPanelFilters);

        jPanelFiltersTab.add(jScrollPaneFilters, java.awt.BorderLayout.CENTER);

        jButtonResetAll.setText("Reset all");
        jButtonResetAll.setToolTipText("Click to reset each modified filters");
        jButtonResetAll.setCursor(new java.awt.Cursor(java.awt.Cursor.HAND_CURSOR));
        jButtonResetAll.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonResetAllActionPerformed(evt);
            }
        });
        jPanelResetAll.add(jButtonResetAll);

        jPanelFiltersTab.add(jPanelResetAll, java.awt.BorderLayout.PAGE_END);

        jTabbedPane.addTab("Filters ", new javax.swing.ImageIcon(getClass().getResource("/icons/view-sort-ascending.png")), jPanelFiltersTab); // NOI18N

        getContentPane().add(jTabbedPane);

        pack();
    }// </editor-fold>//GEN-END:initComponents

    public final void selectProperty(final String property) {
        if (property != null) {
            for (int i = 0; i < propertyNames.length; i++) {
                if (propertyNames[i] == null ? property == null : propertyNames[i].equals(property)) {
                    jComboBoxRenderedProperty.setSelectedIndex(i);
                    return;
                }
            }
        }
    }
    
    /**
     * Set the property names of the current voxel space file.
     * @param names
     */
    public void setPropertyNames(final String[] names) {
        propertyNames = names;

        /* Set the property combobox values */
        jComboBoxRenderedProperty.setModel(new DefaultComboBoxModel(propertyNames));
        jComboBoxRenderedProperty.setSelectedIndex(0);
        
        /* Add a JVoxelFilter for each property */
        for (final String propertyName : propertyNames) {
            /* Create the component */
            final JVoxelFilter filter = new JVoxelFilter();
            
            /* Change the title */
            filter.setTitle(propertyName);
            
            /* Set the min and max value of the property */
            final float min = visualizationFrame.getSpace().
                    getFile().getMinPropertyValue(propertyName);
            final float max = visualizationFrame.getSpace().
                    getFile().getMaxPropertyValue(propertyName);
            filter.setPropertyMinAndMax(min, max);
            
            /* Add a listener */
            filter.addVoxelFilterListener(new JVoxelFilterAdapter() {
                @Override
                public void onChange(JVoxelFilterEvent event) {
                    final Filter f = new Filter(
                            event.getMin(), event.getMax(), event.isNaN());
                    visualizationFrame.getSpace().setFilter(propertyName, f);
                    updateProperty();
                }
            });
            
            /* Add the new JVoxelFilter component to the frame */
            jPanelFilters.add(filter);
            filters.push(filter);
        }
        
        initComponentContents();
    }

    private void initComponentContents() {
        /* Set colorbar names */
        final Field[] fields = ColorGradient.class.getFields();

        try {
            /* Get the gradient fields or the ColorGradient class */
            for (final Field field : fields) {
                final String type = field.getType().getSimpleName();

                if (type.equals("Color[]")) {
                    /* Add each gradient name and colors to the lists */
                    colorbarNameList.add(field.getName());
                    colorbarList.add((Color[]) field.get(ColorGradient.class));
                }
            }
        } catch (IllegalAccessException ex) {
            Logger.getLogger(PropertiesInternalFrame.class.getName()).log(Level.ERROR, null, ex);
        }

        /* Set the colobar combobox values */
        jComboBoxColorBar.setModel(new DefaultComboBoxModel(
                colorbarNameList.toArray(new String[colorbarNameList.size()])));
        jComboBoxColorBar.setSelectedItem("GRADIENT_GREEN_YELLOW_ORANGE_RED");
        
        /* Set the sky combobox values */
        jComboBoxSky.setModel(new DefaultComboBoxModel(Skyboxes.getSkyNames()));
        jComboBoxSky.setSelectedIndex(0);

        /* Set drawing mode slider label */
        final Hashtable t = new Hashtable();
        t.put(0, new JLabel("Point"));
        t.put(10, new JLabel("← Size →"));
        t.put(20, new JLabel("Cube"));
        jSliderDrawingMode.setLabelTable(t);
    }

    /**
     * Update the properties values and the filters.
     */
    private void updateProperty() {
        /* Get the property to render */
        final String renderedProperty
                = (String) jComboBoxRenderedProperty.getSelectedItem();

        if (lockMinAndMax) {
            if (jSliderColorMax.getValue() <= minAndMaxGap) {
                jSliderColorMax.setValue(minAndMaxGap);
            }
            if (jSliderColorMin.getValue() >= (100 - minAndMaxGap)) {
                jSliderColorMin.setValue(100 - minAndMaxGap);
            }
        }
        
        /* Send the property to render */
        visualizationFrame.getSpace().setColorMin((float) jSliderColorMin.getValue() / 100);
        visualizationFrame.getSpace().setColorMax((float) jSliderColorMax.getValue() / 100);
        visualizationFrame.getSpace().setProperty(renderedProperty);
        
        /* Update the min and max property values */
        minValue = visualizationFrame.getSpace().
                    getFile().getMinPropertyValue(renderedProperty);
        maxValue = visualizationFrame.getSpace().
                    getFile().getMaxPropertyValue(renderedProperty);
        
        /* Update the min and max text field content */
        updateMinTextField();
        updateMaxTextField();
    }
    
    private void updateMinTextField() {
        if (lockMinAndMax) {
            jSliderColorMax.setValue(jSliderColorMin.getValue() + minAndMaxGap);
        }
        
        final float value = (float) jSliderColorMin.getValue() / 100;
        float min = minValue + (maxValue - minValue) * value;
        jTextFieldColorMin.setText(Float.toString(min));
    }
    
    private void updateMaxTextField() {
        if (lockMinAndMax) {
            jSliderColorMin.setValue(jSliderColorMax.getValue() - minAndMaxGap);
        }
        
        final float value = (float) jSliderColorMax.getValue() / 100;
        float max = minValue + (maxValue - minValue) * value;
        jTextFieldColorMax.setText(Float.toString(max));
    }

    /**
     * Called when the user change the current rendered property.
     * @param evt
     */
    private void jComboBoxRenderedPropertyActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jComboBoxRenderedPropertyActionPerformed
        updateProperty();
    }//GEN-LAST:event_jComboBoxRenderedPropertyActionPerformed

    private void jComboBoxColorBarActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jComboBoxColorBarActionPerformed
        final Color[] colorbar = colorbarList.get(jComboBoxColorBar.getSelectedIndex());
        visualizationFrame.getSpace().setColorBar(colorbar);
        updateProperty();
    }//GEN-LAST:event_jComboBoxColorBarActionPerformed

    private void jSliderDrawingModeStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_jSliderDrawingModeStateChanged
        visualizationFrame.getSpace().setDrawingMode(
                (float) jSliderDrawingMode.getValue() / 20);
    }//GEN-LAST:event_jSliderDrawingModeStateChanged

    private void jSliderColorMinStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_jSliderColorMinStateChanged
        updateMinTextField();
    }//GEN-LAST:event_jSliderColorMinStateChanged

    private void jSliderColorMaxStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_jSliderColorMaxStateChanged
        updateMaxTextField();
    }//GEN-LAST:event_jSliderColorMaxStateChanged

    private void jSliderColorMinMousePressed(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jSliderColorMinMousePressed
        jSliderColorMin.setCursor(new Cursor(Cursor.MOVE_CURSOR));
    }//GEN-LAST:event_jSliderColorMinMousePressed

    private void jSliderColorMinMouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jSliderColorMinMouseReleased
        jSliderColorMin.setCursor(new Cursor(Cursor.HAND_CURSOR));
        updateProperty();
    }//GEN-LAST:event_jSliderColorMinMouseReleased

    private void jSliderColorMaxMousePressed(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jSliderColorMaxMousePressed
        jSliderColorMax.setCursor(new Cursor(Cursor.MOVE_CURSOR));
    }//GEN-LAST:event_jSliderColorMaxMousePressed

    private void jSliderColorMaxMouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jSliderColorMaxMouseReleased
        jSliderColorMax.setCursor(new Cursor(Cursor.HAND_CURSOR));
        updateProperty();
    }//GEN-LAST:event_jSliderColorMaxMouseReleased

    private void jButtonResetAllActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonResetAllActionPerformed
        final ExecutorService executor = Executors.newCachedThreadPool();
        
        filters.forEach((filter) -> {
            executor.execute(() -> {
                filter.reset();
            });
        });
        
        executor.shutdown();
    }//GEN-LAST:event_jButtonResetAllActionPerformed

    private void jCheckBoxLockActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jCheckBoxLockActionPerformed
        minAndMaxGap = jSliderColorMax.getValue() - jSliderColorMin.getValue();
        minAndMaxGap = minAndMaxGap >= 0 ? minAndMaxGap : -minAndMaxGap;
        lockMinAndMax = jCheckBoxLock.isSelected();
    }//GEN-LAST:event_jCheckBoxLockActionPerformed

    private void jComboBoxSkyActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jComboBoxSkyActionPerformed
        final String name = (String) jComboBoxSky.getSelectedItem();
        visualizationFrame.getSpace().setSky(Skyboxes.getSky(name));
    }//GEN-LAST:event_jComboBoxSkyActionPerformed

    // Parent declaration
    private VisualizationFrame visualizationFrame;
    // End of parent declaration

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButtonResetAll;
    private javax.swing.JCheckBox jCheckBoxLock;
    private javax.swing.JComboBox<String> jComboBoxColorBar;
    private javax.swing.JComboBox<String> jComboBoxRenderedProperty;
    private javax.swing.JComboBox<String> jComboBoxSky;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabelColorMax;
    private javax.swing.JLabel jLabelColorMin;
    private javax.swing.JLabel jLabelLock;
    private javax.swing.JLabel jLabelSky;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanelColorMax;
    private javax.swing.JPanel jPanelColorMin;
    private javax.swing.JPanel jPanelColorbar;
    private javax.swing.JPanel jPanelDrawingMode;
    private javax.swing.JPanel jPanelFilters;
    private javax.swing.JPanel jPanelFiltersTab;
    private javax.swing.JPanel jPanelLock;
    private javax.swing.JPanel jPanelPropertyTab;
    private javax.swing.JPanel jPanelRendererProperty;
    private javax.swing.JPanel jPanelResetAll;
    private javax.swing.JPanel jPanelSky;
    private javax.swing.JScrollPane jScrollPaneFilters;
    private javax.swing.JSlider jSliderColorMax;
    private javax.swing.JSlider jSliderColorMin;
    private javax.swing.JSlider jSliderDrawingMode;
    private javax.swing.JTabbedPane jTabbedPane;
    private javax.swing.JTextField jTextFieldColorMax;
    private javax.swing.JTextField jTextFieldColorMin;
    // End of variables declaration//GEN-END:variables
}
